/*
 * Copyright 2002-2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.samples.petclinic.service;

import java.math.BigDecimal;
import java.util.Collection;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.dao.DataAccessException;
import org.springframework.samples.petclinic.model.Owner;
import org.springframework.samples.petclinic.model.Pet;
import org.springframework.samples.petclinic.model.PetType;
import org.springframework.samples.petclinic.model.Vet;
import org.springframework.samples.petclinic.model.Visit;
import org.springframework.samples.petclinic.repository.OwnerRepository;
import org.springframework.samples.petclinic.repository.PetRepository;
import org.springframework.samples.petclinic.repository.VetRepository;
import org.springframework.samples.petclinic.repository.VisitRepository;
import org.springframework.samples.petclinic.util.PriceCalculator;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Mostly used as a facade for all Petclinic controllers
 * Also a placeholder for @Transactional and @Cacheable annotations
 *
 * @author Michael Isvy
 * @author Michael Isvy
 */
@Service
public class ClinicServiceImpl implements ClinicService {

	private static Logger log = LoggerFactory.getLogger(ClinicServiceImpl.class);

    private PriceCalculator calculator = new PriceCalculator();

    private PetRepository petRepository;
    private VetRepository vetRepository;
    private OwnerRepository ownerRepository;
    private VisitRepository visitRepository;
    private ConfirmationService confirmationService;

    public ClinicServiceImpl() {
    }

    @Autowired
	public void setPetRepository(PetRepository petRepository) {
		this.petRepository = petRepository;
	}

    @Autowired
	public void setVetRepository(VetRepository vetRepository) {
		this.vetRepository = vetRepository;
	}

    @Autowired
	public void setOwnerRepository(OwnerRepository ownerRepository) {
		this.ownerRepository = ownerRepository;
	}

    @Autowired
	public void setVisitRepository(VisitRepository visitRepository) {
		this.visitRepository = visitRepository;
	}

    @Autowired
	public void setConfirmationService(ConfirmationService confirmationService) {
		this.confirmationService = confirmationService;
	}

    // ------ public methods -------
    
	@Override
    @Transactional
	public void deleteOwnerById(int id) throws DataAccessException {
		Owner owner = findOwnerById(id);
		if (owner == null) {
			throw new RuntimeException("Could not delete owner with id " + id + " because it doesn't exist in repository"); 
		}
		ownerRepository.delete(owner);
	}

	@Override
    @Transactional(readOnly = true)
    public Owner findOwnerById(int id) throws DataAccessException {
        return ownerRepository.findById(id);
    }

    @Override
    @Transactional(readOnly = true)
    public Collection<Owner> findOwnerByLastName(String lastName) throws DataAccessException {
        return ownerRepository.findByLastName(lastName);
    }

    @Override
    @Transactional(readOnly = true)
    public Collection<Owner> findOwners() throws DataAccessException {
        return ownerRepository.findAll();
    }

    @Override
    @Transactional(readOnly = true)
    public Pet findPetById(int id) throws DataAccessException {
        return petRepository.findById(id);
    }

    @Override
    @Transactional(readOnly = true)
    public Collection<PetType> findPetTypes() throws DataAccessException {
        return petRepository.findPetTypes();
    }

    @Override
    @Transactional(readOnly = true)
    @Cacheable(value = "vets")
    public Collection<Vet> findVets() throws DataAccessException {
        return vetRepository.findAll();
    }

    @Override
    @Transactional(readOnly = true)
    public Visit findVisitById(int id) throws DataAccessException {
        return visitRepository.findById(id);
    }
    
    @Override
    @Transactional
    public void saveOwner(Owner owner) throws DataAccessException {
        ownerRepository.save(owner);
    }

    @Override
    @Transactional
    public void savePet(Pet pet) throws DataAccessException {
        petRepository.save(pet);
    }
    
    @Override
    @Transactional
    public void saveVisit(Visit visit) throws DataAccessException {
        calculatePrice(visit);
        visitRepository.save(visit);
        try {
			confirmationService.sendConfirmationMessage(visit);
		} catch (Throwable t) {
			log.error("Failed to send confirmation message: " + t.getMessage());
		}
    }

    // ------ private methods -------
    
    private void calculatePrice(Visit visit) {
        BigDecimal price = calculator.calculate(visit.getDate(), visit.getPet());
        visit.setPrice(price);
    }

}
